@@ -7,7 +7,8 @@ use crate::rule_core::{RuleCore, RuleCoreError, SerializableRuleCore};
 
 use ast_grep_core::language::Language;
 use ast_grep_core::replacer::Replacer;
-use ast_grep_core::{Matcher, NodeMatch, StrDoc};
+use ast_grep_core::source::Content;
+use ast_grep_core::{Doc, Matcher, NodeMatch};
 
 use schemars::{gen::SchemaGenerator, schema::Schema, JsonSchema};
 use serde::{Deserialize, Serialize};
@@ -116,7 +117,7 @@ impl JsonSchema for Metadata {
 }
 
 impl<L: Language> SerializableRuleConfig<L> {
-  pub fn get_matcher(&self, globals: &GlobalRules<L>) -> Result<RuleCore<L>, RuleConfigError> {
+  pub fn get_matcher(&self, globals: &GlobalRules) -> Result<RuleCore, RuleConfigError> {
     // every RuleConfig has one rewriters, and the rewriter is shared between sub-rules
     // all RuleConfigs has one common globals
     // every sub-rule has one util
@@ -128,7 +129,7 @@ impl<L: Language> SerializableRuleConfig<L> {
 
   fn register_rewriters(
     &self,
-    rule: &RuleCore<L>,
+    rule: &RuleCore,
     env: DeserializeEnv<L>,
   ) -> Result<(), RuleConfigError> {
     let Some(ser) = &self.rewriters else {
@@ -166,13 +167,13 @@ impl<L: Language> DerefMut for SerializableRuleConfig<L> {
 
 pub struct RuleConfig<L: Language> {
   inner: SerializableRuleConfig<L>,
-  pub matcher: RuleCore<L>,
+  pub matcher: RuleCore,
 }
 
 impl<L: Language> RuleConfig<L> {
   pub fn try_from(
     inner: SerializableRuleConfig<L>,
-    globals: &GlobalRules<L>,
+    globals: &GlobalRules,
   ) -> Result<Self, RuleConfigError> {
     let matcher = inner.get_matcher(globals)?;
     if matcher.potential_kinds().is_none() {
@@ -183,7 +184,7 @@ impl<L: Language> RuleConfig<L> {
 
   pub fn deserialize<'de>(
     deserializer: Deserializer<'de>,
-    globals: &GlobalRules<L>,
+    globals: &GlobalRules,
   ) -> Result<Self, RuleConfigError>
   where
     L: Deserialize<'de>,
@@ -192,13 +193,16 @@ impl<L: Language> RuleConfig<L> {
     Self::try_from(inner, globals)
   }
 
-  pub fn get_message(&self, node: &NodeMatch<StrDoc<L>>) -> String {
+  pub fn get_message<D>(&self, node: &NodeMatch<D>) -> String
+  where
+    D: Doc,
+  {
     let env = self.matcher.get_env(self.language.clone());
     let parsed = Fixer::with_transform(&self.message, &env, &self.transform).expect("should work");
     let bytes = parsed.generate_replacement(node);
-    String::from_utf8(bytes).expect("replacement must be valid utf-8")
+    <D::Source as Content>::encode_bytes(&bytes).to_string()
   }
-  pub fn get_fixer(&self) -> Result<Option<Fixer<L>>, RuleConfigError> {
+  pub fn get_fixer(&self) -> Result<Option<Fixer>, RuleConfigError> {
     if let Some(fix) = &self.fix {
       let env = self.matcher.get_env(self.language.clone());
       let parsed = Fixer::parse(fix, &env, &self.transform).map_err(RuleCoreError::Fixer)?;
@@ -227,6 +231,7 @@ mod test {
   use crate::from_str;
   use crate::rule::SerializableRule;
   use crate::test::TypeScript;
+  use ast_grep_core::tree_sitter::LanguageExt;
 
   fn ts_rule_config(rule: SerializableRule) -> SerializableRuleConfig<TypeScript> {
     let core = SerializableRuleCore {
