@@ -1,9 +1,8 @@
-use crate::language::Language;
-use crate::matcher::{FindAllNodes, Matcher, MatcherExt, NodeMatch};
+use crate::matcher::{Matcher, MatcherExt, NodeMatch};
 use crate::replacer::Replacer;
-use crate::source::{perform_edit, Content, Edit as E, TSParseError};
-use crate::traversal::{Pre, Visitor};
-use crate::{Doc, StrDoc};
+use crate::source::{Content, Edit as E, SgNode};
+use crate::Doc;
+use crate::Language;
 
 type Edit<D> = E<<D as Doc>::Source>;
 
@@ -24,7 +23,7 @@ pub struct Position {
 }
 
 impl Position {
-  fn new(line: usize, byte_column: usize, byte_offset: usize) -> Self {
+  pub fn new(line: usize, byte_column: usize, byte_offset: usize) -> Self {
     Self {
       line,
       byte_column,
@@ -35,13 +34,12 @@ impl Position {
     self.line
   }
   /// TODO: return unicode character offset
-  pub fn column<D: Doc>(&self, node: &Node<D>) -> usize {
-    let source = node.root.doc.get_source();
+  pub fn column<D: Doc>(&self, node: &Node<'_, D>) -> usize {
+    let source = node.get_doc().get_source();
     source.get_char_column(self.byte_column, self.byte_offset)
   }
-  /// Convert to tree-sitter's Point
-  pub fn ts_point(&self) -> tree_sitter::Point {
-    tree_sitter::Point::new(self.line as u32, self.byte_column as u32)
+  pub fn byte_point(&self) -> (u32, u32) {
+    (self.line as u32, self.byte_column as u32)
   }
 }
 
@@ -49,36 +47,12 @@ impl Position {
 /// Note: Root is generic against [`Language`](crate::language::Language)
 #[derive(Clone)]
 pub struct Root<D: Doc> {
-  pub(crate) inner: tree_sitter::Tree,
   pub(crate) doc: D,
 }
 
-impl<L: Language> Root<StrDoc<L>> {
-  pub fn str(src: &str, lang: L) -> Self {
-    Self::try_new(src, lang).expect("should parse")
-  }
-  pub fn get_text(&self) -> &str {
-    &self.doc.src
-  }
-}
-
 impl<D: Doc> Root<D> {
-  pub fn try_new(src: &str, lang: D::Lang) -> Result<Self, TSParseError> {
-    let doc = D::from_str(src, lang);
-    let inner = doc.parse(None)?;
-    Ok(Self { inner, doc })
-  }
-
-  pub fn new(src: &str, lang: D::Lang) -> Self {
-    Self::try_new(src, lang).expect("should parse")
-  }
-  pub fn try_doc(doc: D) -> Result<Self, TSParseError> {
-    let inner = doc.parse(None)?;
-    Ok(Self { inner, doc })
-  }
-
   pub fn doc(doc: D) -> Self {
-    Self::try_doc(doc).expect("Parse doc error")
+    Self { doc }
   }
 
   pub fn lang(&self) -> &D::Lang {
@@ -87,33 +61,45 @@ impl<D: Doc> Root<D> {
   /// The root node represents the entire source
   pub fn root(&self) -> Node<D> {
     Node {
-      inner: self.inner.root_node(),
+      inner: self.doc.root_node(),
       root: self,
     }
   }
 
   // extract non generic implementation to reduce code size
-  pub fn do_edit(&mut self, edit: Edit<D>) -> Result<(), TSParseError> {
-    let source = self.doc.get_source_mut();
-    let input_edit = perform_edit(&mut self.inner, source, &edit);
-    self.inner.edit(&input_edit);
-    self.inner = self.doc.parse(Some(&self.inner))?;
-    Ok(())
+  pub fn edit(&mut self, edit: Edit<D>) -> Result<&mut Self, String> {
+    self.doc.do_edit(&edit)?;
+    Ok(self)
+  }
+
+  pub fn replace<M: Matcher, R: Replacer<D>>(
+    &mut self,
+    pattern: M,
+    replacer: R,
+  ) -> Result<bool, String> {
+    let root = self.root();
+    if let Some(edit) = root.replace(pattern, replacer) {
+      drop(root); // rust cannot auto drop root if D is not specified
+      self.edit(edit)?;
+      Ok(true)
+    } else {
+      Ok(false)
+    }
   }
 
   /// Adopt the tree_sitter as the descendant of the root and return the wrapped sg Node.
   /// It assumes `inner` is the under the root and will panic at dev build if wrong node is used.
-  pub fn adopt<'r>(&'r self, inner: tree_sitter::Node<'r>) -> Node<'r, D> {
+  pub fn adopt<'r>(&'r self, inner: D::Node<'r>) -> Node<'r, D> {
     debug_assert!(self.check_lineage(&inner));
     Node { inner, root: self }
   }
 
-  fn check_lineage(&self, inner: &tree_sitter::Node<'_>) -> bool {
+  fn check_lineage(&self, inner: &D::Node<'_>) -> bool {
     let mut node = inner.clone();
     while let Some(n) = node.parent() {
       node = n;
     }
-    node == self.inner.root_node()
+    node.node_id() == self.doc.root_node().node_id()
   }
 
   /// P.S. I am your father.
@@ -122,78 +108,33 @@ impl<D: Doc> Root<D> {
     debug_assert!(self.check_lineage(&node.inner));
     node.root = self;
   }
-
-  pub fn get_injections<F: Fn(&str) -> Option<D::Lang>>(&self, get_lang: F) -> Vec<Root<D>> {
-    let root = self.root();
-    let range = self.lang().extract_injections(root);
-    let roots = range
-      .into_iter()
-      .filter_map(|(lang, ranges)| {
-        let lang = get_lang(&lang)?;
-        let source = self.doc.get_source();
-        let mut parser = tree_sitter::Parser::new().ok()?;
-        parser.set_included_ranges(&ranges).ok()?;
-        parser.set_language(&lang.get_ts_language()).ok()?;
-        let tree = source.parse_tree_sitter(&mut parser, None).ok()?;
-        tree.map(|t| Self {
-          inner: t,
-          doc: self.doc.clone_with_lang(lang),
-        })
-      })
-      .collect();
-    roots
-  }
 }
 
+// why we need one more content? https://github.com/ast-grep/ast-grep/issues/1951
 /// 'r represents root lifetime
 #[derive(Clone)]
 pub struct Node<'r, D: Doc> {
-  pub(crate) inner: tree_sitter::Node<'r>,
+  pub(crate) inner: D::Node<'r>,
   pub(crate) root: &'r Root<D>,
 }
 pub type KindId = u16;
 
-struct NodeWalker<'tree, D: Doc> {
-  cursor: tree_sitter::TreeCursor<'tree>,
-  root: &'tree Root<D>,
-  count: usize,
-}
-
-impl<'tree, D: Doc> Iterator for NodeWalker<'tree, D> {
-  type Item = Node<'tree, D>;
-  fn next(&mut self) -> Option<Self::Item> {
-    if self.count == 0 {
-      return None;
-    }
-    let ret = Some(Node {
-      inner: self.cursor.node(),
-      root: self.root,
-    });
-    self.cursor.goto_next_sibling();
-    self.count -= 1;
-    ret
-  }
-}
-
-impl<D: Doc> ExactSizeIterator for NodeWalker<'_, D> {
-  fn len(&self) -> usize {
-    self.count
-  }
-}
-
 /// APIs for Node inspection
 impl<'r, D: Doc> Node<'r, D> {
+  pub fn get_doc(&self) -> &'r D {
+    &self.root.doc
+  }
   pub fn node_id(&self) -> usize {
-    self.inner.id()
+    self.inner.node_id()
   }
   pub fn is_leaf(&self) -> bool {
-    self.inner.child_count() == 0
+    self.inner.is_leaf()
   }
   /// if has no named children.
   /// N.B. it is different from is_named && is_leaf
   // see https://github.com/ast-grep/ast-grep/issues/276
   pub fn is_named_leaf(&self) -> bool {
-    self.inner.named_child_count() == 0
+    self.inner.is_named_leaf()
   }
   pub fn is_error(&self) -> bool {
     self.inner.is_error()
@@ -208,93 +149,39 @@ impl<'r, D: Doc> Node<'r, D> {
   pub fn is_named(&self) -> bool {
     self.inner.is_named()
   }
-
-  /// the underlying tree-sitter Node
-  pub fn get_ts_node(&self) -> tree_sitter::Node<'r> {
-    self.inner.clone()
+  pub fn is_missing(&self) -> bool {
+    self.inner.is_missing()
   }
 
   /// byte offsets of start and end.
   pub fn range(&self) -> std::ops::Range<usize> {
-    (self.inner.start_byte() as usize)..(self.inner.end_byte() as usize)
+    self.inner.range()
   }
 
   /// Nodes' start position in terms of zero-based rows and columns.
   pub fn start_pos(&self) -> Position {
-    let pos = self.inner.start_position();
-    let byte = self.inner.start_byte() as usize;
-    Position::new(pos.row() as usize, pos.column() as usize, byte)
+    self.inner.start_pos()
   }
 
   /// Nodes' end position in terms of rows and columns.
   pub fn end_pos(&self) -> Position {
-    let pos = self.inner.end_position();
-    let byte = self.inner.end_byte() as usize;
-    Position::new(pos.row() as usize, pos.column() as usize, byte)
+    self.inner.end_pos()
   }
 
   pub fn text(&self) -> Cow<'r, str> {
-    let source = self.root.doc.get_source();
-    source.get_text(&self.inner)
-  }
-
-  /// Node's tree structure dumped in Lisp like S-expression
-  pub fn to_sexp(&self) -> Cow<'_, str> {
-    self.inner.to_sexp()
+    self.root.doc.get_node_text(&self.inner)
   }
 
   pub fn lang(&self) -> &'r D::Lang {
     self.root.lang()
   }
-}
 
-// TODO: figure out how to do this
-impl<'r, L: Language> Node<'r, StrDoc<L>> {
-  #[doc(hidden)]
-  pub fn display_context(&self, before: usize, after: usize) -> DisplayContext<'r> {
-    let source = self.root.doc.get_source().as_str();
-    let bytes = source.as_bytes();
-    let start = self.inner.start_byte() as usize;
-    let end = self.inner.end_byte() as usize;
-    let (mut leading, mut trailing) = (start, end);
-    let mut lines_before = before + 1;
-    while leading > 0 {
-      if bytes[leading - 1] == b'\n' {
-        lines_before -= 1;
-        if lines_before == 0 {
-          break;
-        }
-      }
-      leading -= 1;
-    }
-    let mut lines_after = after + 1;
-    // tree-sitter will append line ending to source so trailing can be out of bound
-    trailing = trailing.min(bytes.len());
-    while trailing < bytes.len() {
-      if bytes[trailing] == b'\n' {
-        lines_after -= 1;
-        if lines_after == 0 {
-          break;
-        }
-      }
-      trailing += 1;
-    }
-    // lines_before means we matched all context, offset is `before` itself
-    let offset = if lines_before == 0 {
-      before
-    } else {
-      // otherwise, there are fewer than `before` line in src, compute the actual line
-      before + 1 - lines_before
-    };
-    DisplayContext {
-      matched: self.text(),
-      leading: &source[leading..start],
-      trailing: &source[end..trailing],
-      start_line: self.start_pos().line() - offset,
-    }
+  /// the underlying tree-sitter Node
+  pub fn get_inner_node(&self) -> D::Node<'r> {
+    self.inner.clone()
   }
 
-  pub fn root(&self) -> &'r Root<StrDoc<L>> {
+  pub fn root(&self) -> &'r Root<D> {
     self.root
   }
 }
@@ -303,38 +190,27 @@ impl<'r, L: Language> Node<'r, StrDoc<L>> {
  * Corresponds to inside/has/precedes/follows
  */
 impl<D: Doc> Node<'_, D> {
-  pub fn matches<M: Matcher<D::Lang>>(&self, m: M) -> bool {
+  pub fn matches<M: Matcher>(&self, m: M) -> bool {
     m.match_node(self.clone()).is_some()
   }
 
-  pub fn inside<M: Matcher<D::Lang>>(&self, m: M) -> bool {
+  pub fn inside<M: Matcher>(&self, m: M) -> bool {
     self.ancestors().find_map(|n| m.match_node(n)).is_some()
   }
 
-  pub fn has<M: Matcher<D::Lang>>(&self, m: M) -> bool {
+  pub fn has<M: Matcher>(&self, m: M) -> bool {
     self.dfs().skip(1).find_map(|n| m.match_node(n)).is_some()
   }
 
-  pub fn precedes<M: Matcher<D::Lang>>(&self, m: M) -> bool {
+  pub fn precedes<M: Matcher>(&self, m: M) -> bool {
     self.next_all().find_map(|n| m.match_node(n)).is_some()
   }
 
-  pub fn follows<M: Matcher<D::Lang>>(&self, m: M) -> bool {
+  pub fn follows<M: Matcher>(&self, m: M) -> bool {
     self.prev_all().find_map(|n| m.match_node(n)).is_some()
   }
 }
 
-pub struct DisplayContext<'r> {
-  /// content for the matched node
-  pub matched: Cow<'r, str>,
-  /// content before the matched node
-  pub leading: &'r str,
-  /// content after the matched node
-  pub trailing: &'r str,
-  /// zero-based start line of the context
-  pub start_line: usize,
-}
-
 /// tree traversal API
 impl<'r, D: Doc> Node<'r, D> {
   #[must_use]
@@ -346,20 +222,16 @@ impl<'r, D: Doc> Node<'r, D> {
     })
   }
 
-  pub fn children<'s>(&'s self) -> impl ExactSizeIterator<Item = Node<'r, D>> + 's {
-    let mut cursor = self.inner.walk();
-    cursor.goto_first_child();
-    NodeWalker {
-      cursor,
+  pub fn children(&self) -> impl ExactSizeIterator<Item = Node<'r, D>> + '_ {
+    self.inner.children().map(|inner| Node {
+      inner,
       root: self.root,
-      count: self.inner.child_count() as usize,
-    }
+    })
   }
 
   #[must_use]
   pub fn child(&self, nth: usize) -> Option<Self> {
-    // TODO: support usize
-    let inner = self.inner.child(nth as u32)?;
+    let inner = self.inner.child(nth)?;
     Some(Node {
       inner,
       root: self.root,
@@ -367,7 +239,7 @@ impl<'r, D: Doc> Node<'r, D> {
   }
 
   pub fn field(&self, name: &str) -> Option<Self> {
-    let inner = self.inner.child_by_field_name(name)?;
+    let inner = self.inner.field(name)?;
     Some(Node {
       inner,
       root: self.root,
@@ -382,55 +254,26 @@ impl<'r, D: Doc> Node<'r, D> {
     })
   }
 
-  pub fn field_children(&self, name: &str) -> impl Iterator<Item = Node<'r, D>> {
-    let field_id = self.root.lang().get_ts_language().field_id_for_name(name);
-    let root = self.root;
-    let mut cursor = self.inner.walk();
-    cursor.goto_first_child();
-    // if field_id is not found, iteration is done
-    let mut done = field_id.is_none();
-
-    std::iter::from_fn(move || {
-      if done {
-        return None;
-      }
-      while cursor.field_id() != field_id {
-        if !cursor.goto_next_sibling() {
-          return None;
-        }
-      }
-      let inner = cursor.node();
-      if !cursor.goto_next_sibling() {
-        done = true;
-      }
-      Some(Node { inner, root })
+  pub fn field_children(&self, name: &str) -> impl Iterator<Item = Node<'r, D>> + '_ {
+    let field_id = self.lang().field_to_id(name);
+    self.inner.field_children(field_id).map(|inner| Node {
+      inner,
+      root: self.root,
     })
   }
 
   /// Returns all ancestors nodes of `self`.
   /// Using cursor is overkill here because adjust cursor is too expensive.
   pub fn ancestors(&self) -> impl Iterator<Item = Node<'r, D>> + '_ {
-    let mut ancestor = Some(self.root.root().inner);
-    let self_id = self.inner.id();
-    std::iter::from_fn(move || {
-      let inner = ancestor.take()?;
-      if inner.id() == self_id {
-        return None;
-      }
-      ancestor = inner.child_with_descendant(self.inner.clone());
-      Some(Node {
-        inner,
-        root: self.root,
-      })
+    let root = self.root.doc.root_node();
+    self.inner.ancestors(root).map(|inner| Node {
+      inner,
+      root: self.root,
     })
-    // We must iterate up the tree to preserve backwards compatibility
-    .collect::<Vec<_>>()
-    .into_iter()
-    .rev()
   }
   #[must_use]
   pub fn next(&self) -> Option<Self> {
-    let inner = self.inner.next_sibling()?;
+    let inner = self.inner.next()?;
     Some(Node {
       inner,
       root: self.root,
@@ -441,108 +284,65 @@ impl<'r, D: Doc> Node<'r, D> {
   // NOTE: Need go to parent first, then move to current node by byte offset.
   // This is because tree_sitter cursor is scoped to the starting node.
   // See https://github.com/tree-sitter/tree-sitter/issues/567
-  #[cfg(not(target_arch = "wasm32"))]
   pub fn next_all(&self) -> impl Iterator<Item = Node<'r, D>> + '_ {
-    // if root is none, use self as fallback to return a type-stable Iterator
-    let node = self.parent().unwrap_or_else(|| self.clone());
-    let mut cursor = node.inner.walk();
-    cursor.goto_first_child_for_byte(self.inner.start_byte());
-    std::iter::from_fn(move || {
-      if cursor.goto_next_sibling() {
-        Some(self.root.adopt(cursor.node()))
-      } else {
-        None
-      }
-    })
-  }
-
-  // wasm32 has wrong goto_first_child_for_byte
-  #[cfg(target_arch = "wasm32")]
-  pub fn next_all(&self) -> impl Iterator<Item = Node<'r, D>> + '_ {
-    let mut node = self.clone();
-    std::iter::from_fn(move || {
-      node.next().map(|n| {
-        node = n.clone();
-        n
-      })
+    self.inner.next_all().map(|inner| Node {
+      inner,
+      root: self.root,
     })
   }
 
   #[must_use]
   pub fn prev(&self) -> Option<Node<'r, D>> {
-    let inner = self.inner.prev_sibling()?;
+    let inner = self.inner.prev()?;
     Some(Node {
       inner,
       root: self.root,
     })
   }
 
-  #[cfg(not(target_arch = "wasm32"))]
   pub fn prev_all(&self) -> impl Iterator<Item = Node<'r, D>> + '_ {
-    // if root is none, use self as fallback to return a type-stable Iterator
-    let node = self.parent().unwrap_or_else(|| self.clone());
-    let mut cursor = node.inner.walk();
-    cursor.goto_first_child_for_byte(self.inner.start_byte());
-    std::iter::from_fn(move || {
-      if cursor.goto_previous_sibling() {
-        Some(self.root.adopt(cursor.node()))
-      } else {
-        None
-      }
+    self.inner.prev_all().map(|inner| Node {
+      inner,
+      root: self.root,
     })
   }
 
-  // wasm32 has wrong goto_first_child_for_byte
-  #[cfg(target_arch = "wasm32")]
-  pub fn prev_all(&self) -> impl Iterator<Item = Node<'r, D>> + '_ {
-    let mut node = self.clone();
-    std::iter::from_fn(move || {
-      node.prev().map(|n| {
-        node = n.clone();
-        n
-      })
+  pub fn dfs<'s>(&'s self) -> impl Iterator<Item = Node<'r, D>> + 's {
+    self.inner.dfs().map(|inner| Node {
+      inner,
+      root: self.root,
     })
   }
 
-  pub fn dfs<'s>(&'s self) -> Pre<'r, D> {
-    Pre::new(self)
-  }
-
   #[must_use]
-  pub fn find<M: Matcher<D::Lang>>(&self, pat: M) -> Option<NodeMatch<'r, D>> {
+  pub fn find<M: Matcher>(&self, pat: M) -> Option<NodeMatch<'r, D>> {
     pat.find_node(self.clone())
   }
 
-  pub fn find_all<M: Matcher<D::Lang>>(&self, pat: M) -> impl Iterator<Item = NodeMatch<'r, D>> {
-    FindAllNodes::new(pat, self.clone())
+  pub fn find_all<'s, M: Matcher + 's>(
+    &'s self,
+    pat: M,
+  ) -> impl Iterator<Item = NodeMatch<'r, D>> + 's {
+    let kinds = pat.potential_kinds();
+    self.dfs().filter_map(move |cand| {
+      if let Some(k) = &kinds {
+        if !k.contains(cand.kind_id().into()) {
+          return None;
+        }
+      }
+      pat.match_node(cand)
+    })
   }
 }
 
 /// Tree manipulation API
 impl<D: Doc> Node<'_, D> {
-  pub fn replace<M: Matcher<D::Lang>, R: Replacer<D>>(
-    &self,
-    matcher: M,
-    replacer: R,
-  ) -> Option<Edit<D>> {
+  pub fn replace<M: Matcher, R: Replacer<D>>(&self, matcher: M, replacer: R) -> Option<Edit<D>> {
     let matched = matcher.find_node(self.clone())?;
     let edit = matched.make_edit(&matcher, &replacer);
     Some(edit)
   }
 
-  pub fn replace_all<M: Matcher<D::Lang>, R: Replacer<D>>(
-    &self,
-    matcher: M,
-    replacer: R,
-  ) -> Vec<Edit<D>> {
-    // TODO: support nested matches like Some(Some(1)) with pattern Some($A)
-    Visitor::new(&matcher)
-      .reentrant(false)
-      .visit(self.clone())
-      .map(|matched| matched.make_edit(&matcher, &replacer))
-      .collect()
-  }
-
   pub fn after(&self) -> Edit<D> {
     todo!()
   }
@@ -582,6 +382,7 @@ impl<D: Doc> Node<'_, D> {
 #[cfg(test)]
 mod test {
   use crate::language::{Language, Tsx};
+  use crate::tree_sitter::LanguageExt;
   #[test]
   fn test_is_leaf() {
     let root = Tsx.ast_grep("let a = 123");
@@ -724,7 +525,7 @@ if (a) {
     let root = Tsx.ast_grep("class A{}");
     let root = root.root();
     let node = root.find("class $C {}").expect("should exist");
-    let id = Tsx.get_ts_language().field_id_for_name("name").unwrap();
+    let id = Tsx.field_to_id("name").unwrap();
     assert!(node.child_by_field_id(id).is_some());
     assert!(node.child_by_field_id(id + 1).is_none());
   }
@@ -745,9 +546,9 @@ if (a) {
     let root = root.root();
     let node = root.find("$A").expect("should exist");
     assert_eq!(node.start_pos().line(), 0);
-    assert_eq!(node.start_pos().column(&node), 0);
+    assert_eq!(node.start_pos().column(&*node), 0);
     assert_eq!(node.end_pos().line(), 0);
-    assert_eq!(node.end_pos().column(&node), 1);
+    assert_eq!(node.end_pos().column(&*node), 1);
   }
 
   #[test]
@@ -756,15 +557,15 @@ if (a) {
     let root = root.root();
     let node = root.find("$A").expect("should exist");
     assert_eq!(node.start_pos().line(), 0);
-    assert_eq!(node.start_pos().column(&node), 0);
+    assert_eq!(node.start_pos().column(&*node), 0);
     assert_eq!(node.end_pos().line(), 0);
-    assert_eq!(node.end_pos().column(&node), 1);
+    assert_eq!(node.end_pos().column(&*node), 1);
     let root = Tsx.ast_grep("\n  🦀🦀");
     let root = root.root();
     let node = root.find("$A").expect("should exist");
     assert_eq!(node.start_pos().line(), 1);
-    assert_eq!(node.start_pos().column(&node), 2);
+    assert_eq!(node.start_pos().column(&*node), 2);
     assert_eq!(node.end_pos().line(), 1);
-    assert_eq!(node.end_pos().column(&node), 4);
+    assert_eq!(node.end_pos().column(&*node), 4);
   }
 }
