@@ -1,12 +1,12 @@
 use crate::napi_lang::NapiLang;
 
 use ast_grep_config::{DeserializeEnv, RuleCore, SerializableRuleCore};
-use ast_grep_core::source::{Content, Doc, Edit, TSParseError};
-use ast_grep_core::Language;
+use ast_grep_core::source::{Content, Doc, Edit};
+use ast_grep_core::tree_sitter::{ContentExt, LanguageExt, TSParseError};
 use napi::anyhow::Error;
 use napi::bindgen_prelude::Result as NapiResult;
 use napi_derive::napi;
-use tree_sitter::{InputEdit, Node, Parser, ParserError, Point, Tree};
+use tree_sitter::{InputEdit, Node, Parser, Point, Tree};
 
 use std::borrow::Cow;
 use std::ops::Range;
@@ -29,7 +29,7 @@ pub struct NapiConfig {
 }
 
 impl NapiConfig {
-  pub fn parse_with(self, lang: NapiLang) -> NapiResult<RuleCore<NapiLang>> {
+  pub fn parse_with(self, lang: NapiLang) -> NapiResult<RuleCore> {
     let rule = SerializableRuleCore {
       rule: serde_json::from_value(self.rule)?,
       constraints: self.constraints.map(serde_json::from_value).transpose()?,
@@ -55,19 +55,28 @@ pub struct Wrapper {
 
 impl Content for Wrapper {
   type Underlying = u16;
-  fn parse_tree_sitter(
-    &self,
-    parser: &mut Parser,
-    tree: Option<&Tree>,
-  ) -> std::result::Result<Option<Tree>, ParserError> {
-    parser.parse_utf16_le(self.inner.as_slice(), tree)
-  }
   fn get_range(&self, range: Range<usize>) -> &[Self::Underlying] {
     // the range is in byte offset, but our underlying is u16
     let start = range.start / 2;
     let end = range.end / 2;
     &self.inner.as_slice()[start..end]
   }
+  fn decode_str(src: &str) -> Cow<[Self::Underlying]> {
+    let v: Vec<_> = src.encode_utf16().collect();
+    Cow::Owned(v)
+  }
+
+  fn encode_bytes(bytes: &[Self::Underlying]) -> Cow<str> {
+    let s = String::from_utf16_lossy(bytes);
+    Cow::Owned(s)
+  }
+  fn get_char_column(&self, column: usize, _offset: usize) -> usize {
+    // utf-16 is 2 bytes per character, this is O(1) operation!
+    column / 2
+  }
+}
+
+impl ContentExt for Wrapper {
   fn accept_edit(&mut self, edit: &Edit<Self>) -> InputEdit {
     let start_byte = edit.position;
     let old_end_byte = edit.position + edit.deleted_length;
@@ -86,26 +95,6 @@ impl Content for Wrapper {
       &new_end_position,
     )
   }
-  fn get_text<'a>(&'a self, node: &Node) -> Cow<'a, str> {
-    let slice = self.inner.as_slice();
-    let start = node.start_byte() as usize / 2;
-    let end = node.end_byte() as usize / 2;
-    String::from_utf16_lossy(&slice[start..end]).into()
-  }
-
-  fn decode_str(src: &str) -> Cow<[Self::Underlying]> {
-    let v: Vec<_> = src.encode_utf16().collect();
-    Cow::Owned(v)
-  }
-
-  fn encode_bytes(bytes: &[Self::Underlying]) -> Cow<str> {
-    let s = String::from_utf16_lossy(bytes);
-    Cow::Owned(s)
-  }
-  fn get_char_column(&self, column: usize, _offset: usize) -> usize {
-    // utf-16 is 2 bytes per character, this is O(1) operation!
-    column / 2
-  }
 }
 
 fn pos_for_byte_offset(input: &[u16], byte_offset: usize) -> Point {
@@ -127,47 +116,59 @@ fn pos_for_byte_offset(input: &[u16], byte_offset: usize) -> Point {
 pub struct JsDoc {
   lang: NapiLang,
   source: Wrapper,
+  tree: tree_sitter::Tree,
+}
+
+fn parse(
+  source: &Wrapper,
+  lang: &NapiLang,
+  old_tree: Option<&Tree>,
+) -> std::result::Result<Tree, TSParseError> {
+  let mut parser = Parser::new()?;
+  let ts_lang = lang.get_ts_language();
+  parser.set_language(&ts_lang)?;
+  if let Some(tree) = parser.parse_utf16_le(source.inner.as_slice(), old_tree)? {
+    Ok(tree)
+  } else {
+    Err(TSParseError::TreeUnavailable)
+  }
 }
 
 impl JsDoc {
-  pub fn new(src: String, lang: NapiLang) -> Self {
+  pub fn try_new(src: String, lang: NapiLang) -> napi::anyhow::Result<Self> {
     let source = Wrapper {
       inner: src.encode_utf16().collect(),
     };
-    Self { source, lang }
+    let tree = parse(&source, &lang, None)?;
+    Ok(Self { source, lang, tree })
   }
 }
 
 impl Doc for JsDoc {
   type Lang = NapiLang;
   type Source = Wrapper;
-  fn parse(&self, old_tree: Option<&Tree>) -> std::result::Result<Tree, TSParseError> {
-    let mut parser = Parser::new()?;
-    let ts_lang = self.lang.get_ts_language();
-    parser.set_language(&ts_lang)?;
-    if let Some(tree) = self.source.parse_tree_sitter(&mut parser, old_tree)? {
-      Ok(tree)
-    } else {
-      Err(TSParseError::TreeUnavailable)
-    }
-  }
+  type Node<'r> = Node<'r>;
   fn get_lang(&self) -> &Self::Lang {
     &self.lang
   }
   fn get_source(&self) -> &Self::Source {
     &self.source
   }
-  fn get_source_mut(&mut self) -> &mut Self::Source {
-    &mut self.source
+  fn do_edit(&mut self, edit: &Edit<Self::Source>) -> Result<(), String> {
+    let source = &mut self.source;
+    let input_edit = source.accept_edit(edit);
+    self.tree.edit(&input_edit);
+    self.tree = parse(source, &self.lang, Some(&self.tree)).map_err(|e| e.to_string())?;
+    Ok(())
   }
-  fn from_str(src: &str, lang: Self::Lang) -> Self {
-    JsDoc::new(src.into(), lang)
+  fn root_node(&self) -> Node<'_> {
+    self.tree.root_node()
   }
-  fn clone_with_lang(&self, lang: Self::Lang) -> Self {
-    JsDoc {
-      source: self.source.clone(),
-      lang,
-    }
+  fn get_node_text<'a>(&'a self, node: &Node) -> Cow<'a, str> {
+    let slice = self.source.inner.as_slice();
+    let start = node.start_byte() as usize / 2;
+    let end = node.end_byte() as usize / 2;
+    String::from_utf16_lossy(&slice[start..end]).into()
   }
 }
 
@@ -176,9 +177,12 @@ mod test {
   use super::*;
   use ast_grep_core::AstGrep;
   use ast_grep_language::SupportLang;
+  fn make_doc(src: &str) -> JsDoc {
+    JsDoc::try_new(src.to_string(), SupportLang::JavaScript.into()).expect("should work")
+  }
   #[test]
   fn test_js_doc() {
-    let doc = JsDoc::new("console.log(123)".into(), SupportLang::JavaScript.into());
+    let doc = make_doc("console.log(123)");
     let grep = AstGrep::doc(doc);
     assert_eq!(grep.root().text(), "console.log(123)");
     let node = grep.root().find("console");
@@ -187,10 +191,7 @@ mod test {
 
   #[test]
   fn test_js_doc_single_node_replace() {
-    let doc = JsDoc::new(
-      "console.log(1 + 2 + 3)".into(),
-      SupportLang::JavaScript.into(),
-    );
+    let doc = make_doc("console.log(1 + 2 + 3)");
     let mut grep = AstGrep::doc(doc);
     let edit = grep
       .root()
@@ -202,10 +203,7 @@ mod test {
 
   #[test]
   fn test_js_doc_multiple_node_replace() {
-    let doc = JsDoc::new(
-      "console.log(1 + 2 + 3)".into(),
-      SupportLang::JavaScript.into(),
-    );
+    let doc = make_doc("console.log(1 + 2 + 3)");
     let mut grep = AstGrep::doc(doc);
     let edit = grep
       .root()
