@@ -1,9 +1,11 @@
 use anyhow::{bail, Context, Result};
 use ast_grep_config::SerializableRuleConfig;
-use ast_grep_core::{language::TSLanguage, Language};
+use ast_grep_core::matcher::{Pattern, PatternBuilder, PatternError};
+use ast_grep_core::tree_sitter::{LanguageExt, TSLanguage};
+use ast_grep_core::Language;
 use ast_grep_language::{
-  Bash, CSharp, Cpp, Css, Elixir, Go, Haskell, Html, Java, JavaScript, Json, Kotlin, Lua, Php,
-  Python, Ruby, Rust, Scala, Swift, Tsx, TypeScript, Yaml, C,
+  Alias, Bash, CSharp, Cpp, Css, Elixir, Go, Haskell, Html, Java, JavaScript, Json, Kotlin, Lua,
+  Php, Python, Ruby, Rust, Scala, Swift, Tsx, TypeScript, Yaml, C,
 };
 use schemars::{
   gen::SchemaGenerator,
@@ -53,7 +55,7 @@ fn generate_lang_schemas() -> Result<()> {
   generate_lang_schema(Yaml, "yaml")
 }
 
-fn generate_lang_schema<T: Language>(lang: T, name: &str) -> Result<()> {
+fn generate_lang_schema<T: LanguageExt + Alias>(lang: T, name: &str) -> Result<()> {
   let mut schema = schema_for!(SerializableRuleConfig<PlaceholderLang>);
   tweak_schema(&mut schema)?;
   add_lang_info_to_schema(&mut schema, lang, name)?;
@@ -79,7 +81,7 @@ fn tweak_schema(schema: &mut RootSchema) -> Result<()> {
   Ok(())
 }
 
-fn add_lang_info_to_schema<T: Language>(
+fn add_lang_info_to_schema<T: LanguageExt + Alias>(
   schema: &mut RootSchema,
   lang: T,
   name: &str,
@@ -116,6 +118,21 @@ fn add_lang_info_to_schema<T: Language>(
     bail!("kind's type is not object!")
   };
   kind.enum_values = Some(named_nodes);
+  let Schema::Object(language) = definitions
+    .get_mut("Language")
+    .context("must have Language")?
+  else {
+    bail!("Language's type is not an object!")
+  };
+  language.enum_values = Some(
+    T::ALIAS
+      .iter()
+      .map(|alias| serde_json::Value::String(alias.to_string()))
+      .chain(std::iter::once(serde_json::Value::String(format!(
+        "{lang}"
+      ))))
+      .collect(),
+  );
   Ok(())
 }
 
@@ -210,6 +227,17 @@ impl JsonSchema for PlaceholderLang {
 }
 
 impl Language for PlaceholderLang {
+  fn kind_to_id(&self, _kind: &str) -> u16 {
+    unreachable!("PlaceholderLang is only for json schema")
+  }
+  fn field_to_id(&self, _field: &str) -> Option<u16> {
+    unreachable!("PlaceholderLang is only for json schema")
+  }
+  fn build_pattern(&self, _b: &PatternBuilder) -> Result<Pattern, PatternError> {
+    unreachable!("PlaceholderLang is only for json schema")
+  }
+}
+impl LanguageExt for PlaceholderLang {
   fn get_ts_language(&self) -> TSLanguage {
     unreachable!("PlaceholderLang is only for json schema")
   }
